CREATE INDEX person_firstname_upper_idx on persons (upper(firstname));
CREATE INDEX person_surname_upper_idx on persons (upper(surname));
CREATE INDEX person_personid_upper_idx on persons (upper(personid));
CREATE INDEX person_email_upper_idx on persons (upper(email));
