-- ###
-- Copyright (c) 2013, Rice University
-- This software is subject to the provisions of the GNU Affero General
-- Public License version 3 (AGPLv3).
-- See LICENCE.txt for details.
-- ###

INSERT INTO tags VALUES (0, 'Test/Draft', 'internal');
INSERT INTO tags VALUES (1, 'Arts', 'ISKME subject');
INSERT INTO tags VALUES (2, 'Business', 'ISKME subject');
INSERT INTO tags VALUES (3, 'Humanities', 'ISKME subject');
INSERT INTO tags VALUES (4, 'Mathematics and Statistics', 'ISKME subject');
INSERT INTO tags VALUES (5, 'Science and Technology', 'ISKME subject');
INSERT INTO tags VALUES (6, 'Social Sciences', 'ISKME subject');
INSERT INTO tags VALUES (7, 'OpenStax Featured', 'featured');
INSERT INTO tags VALUES (8, 'CNX Featured', 'featured');
